@@ -25,8 +25,11 @@
  * o Integrates source code from Nmap                                      *
  * o Reads or includes Nmap copyrighted data files, such as                *
  *   nmap-os-fingerprints or nmap-service-probes.                          *
- * o Executes Nmap                                                         *
- * o Integrates/includes/aggregates Nmap into an executable installer      *
+ * o Executes Nmap and parses the results (as opposed to typical shell or  *
+ *   execution-menu apps, which simply display raw Nmap output and so are  *
+ *   not derivative works.)                                                * 
+ * o Integrates/includes/aggregates Nmap into a proprietary executable     *
+ *   installer, such as those produced by InstallShield.                   *
  * o Links to a library or executes a program that does any of the above   *
  *                                                                         *
  * The term "Nmap" should be taken to also include any portions or derived *
@@ -53,8 +56,17 @@
  * the continued development of Nmap technology.  Please email             *
  * sales@insecure.com for further information.                             *
  *                                                                         *
+ * As a special exception to the GPL terms, Insecure.Com LLC grants        *
+ * permission to link the code of this program with any version of the     *
+ * OpenSSL library which is distributed under a license identical to that  *
+ * listed in the included Copying.OpenSSL file, and distribute linked      *
+ * combinations including the two. You must obey the GNU GPL in all        *
+ * respects for all of the code used other than OpenSSL.  If you modify    *
+ * this file, you may extend this exception to your version of the file,   *
+ * but you are not obligated to do so.                                     *
+ *                                                                         *
  * If you received these files with a written license agreement or         *
- * contract stating terms other than the (GPL) terms above, then that      *
+ * contract stating terms other than the terms above, then that            *
  * alternative license agreement takes precedence over these comments.     *
  *                                                                         *
  * Source is provided to this software because we believe users have a     *
@@ -80,11 +92,12 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of              *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *
  * General Public License for more details at                              *
- * http://www.gnu.org/copyleft/gpl.html .                                  *
+ * http://www.gnu.org/copyleft/gpl.html , or in the COPYING file included  *
+ * with Nmap.                                                              *
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: charpool.cc,v 1.6 2004/03/12 01:59:04 fyodor Exp $ */
+/* $Id: charpool.cc 3200 2006-03-05 23:59:46Z fyodor $ */
 
 
 /* Character pool memory allocation */
@@ -94,11 +107,13 @@ static char *charpool[16];
 static int currentcharpool;
 static int currentcharpoolsz;
 static char *nextchar;
-static int charpool_initialized = 0;
 
 #define ALIGN_ON sizeof(char *)
 
 static int cp_init(void) {
+  static int charpool_initialized = 0;
+  if (charpool_initialized) return 0;
+
   /* Create our char pool */
   currentcharpool = 0;
   currentcharpoolsz = 16384;
@@ -122,7 +137,7 @@ void *cp_alloc(int sz) {
   char *p;
   int modulus;
 
-  if (!charpool_initialized) cp_init();
+  cp_init();
 
   if ((modulus = sz % ALIGN_ON))
     sz += ALIGN_ON - modulus;
@@ -146,8 +161,7 @@ char *q;
 char *end;
 int modulus;
 
- if (!charpool_initialized) 
-   cp_init();
+ cp_init();
 
  end = charpool[currentcharpool] + currentcharpoolsz;
  q = nextchar;
